require 'sawyer'
require 'ruby-progressbar'
require 'crowdkit/version'
require 'crowdkit/core_ext/hash'
require 'crowdkit/core_ext/array'
require 'crowdkit/core_ext/sawyer'
require 'crowdkit/config'
require 'crowdkit/api'
require 'crowdkit/client'
require 'crowdkit/error'
require 'crowdkit/middleware/raise_error'

module Crowdkit
  def self.new(overrides={}, &block)
    #Use the global config if no overrides present
    if @configuration && overrides.empty? && !block_given?
      Client.new(@configuration)
    else
      Client.new(overrides, &block)
    end
  end

  def self.last_response
    @last_response
  end

  def self.last_response=(last_response)
    @last_response = last_response
  end

  def self.configure(overrides={}, &block)
    @configuration = Config.new(overrides, &block)
  end

  def self.reset!
    @last_response = nil
    @configuration = nil
  end

  # This takes a response from a resource that processes something in the
  # background (i.e. a 202 response) and will ping the status API until it timesout
  # an error is returned, or the background process is completed.
  def self.wait_on_status(resource, max_time_to_wait_in_seconds = 30)
    raise ArgumentError, "This resource did not respond with a 202 status code and has no status to wait on." unless resource.last_response.status == 202
    progressbar = ProgressBar.create(total: 100)
    status = resource.current_status
    wait = 1

    max_time_to_wait_in_seconds.times do |i|
      break if status.state == "completed"
      raise StatusError.new(status) if ["failed", "killed"].include?(status.state)

      #increase wait time to 5 seconds after 5 seconds
      wait = 5 if i == 5
      #increase wait time to 10 seconds after 30 seconds
      wait = 10 if i == 30

      if i % wait == 0
        percents = status.percent_complete.to_i
        progressbar.progress = percents if percents > 0
        sleep(wait)
        status = resource.current_status
      end
    end

    raise StatusError.new(status, true) unless status.state == "completed"
    progressbar.finish
    status
  end

  private

  #Not convinced we even want to support this...
  def self.method_missing(method_name, *args, &block)
    return super unless new.respond_to?(method_name)
    new.send(method_name, *args, &block)
  end
end
