module Crowdkit
  #The top level client, all API access starts here.  It's unlike the other
  #subclasses of API as it overrides the initialize method and sets us up for
  #chaining.
  class Client < API
    attr_accessor :last_response
    attr_reader :config

    #Accepts a hash of configuration overrides or a Config object
    def initialize(overrides, &block)
      @config = overrides.is_a?(Config) ? overrides : Config.new(overrides, &block)
      @client = self
      @stored_params = {}
    end

    def root
      do_get("")
    end

    namespace :workers
    namespace :worksets
    namespace :jobs
    namespace :units
    namespace :statuses
    namespace :judgments
    namespace :account
  end
end

require 'crowdkit/client/workers'
require 'crowdkit/client/worksets'
require 'crowdkit/client/jobs'
require 'crowdkit/client/units'
require 'crowdkit/client/statuses'
require 'crowdkit/client/judgments'
require 'crowdkit/client/poll'
require 'crowdkit/client/account'
