module Crowdkit
  class Client::Jobs < API
    def create(*args)
      arguments(args, required: [:title])

      do_post("jobs", {title: title}.merge(arguments.params))
    end

    def update(*args)
      arguments(args, required: [:job_id])

      do_patch("jobs/#{job_id}", arguments.params)
    end

    def get(*args)
      arguments(args, required: [:job_id])

      do_get("jobs/#{job_id}")
    end
    alias :find :get
    alias :show :get

    def search(*args)
      arguments(args)
      params = arguments.params
      params[:query] = args.first if args.first.is_a?(String)

      response = do_get("jobs", query: arguments.params)
      return response unless block_given?
      response.each { |el| yield el }
    end
    alias :list :search
    alias :all :search

    [:statuses, :judgments].each do |method|
      define_method(method) do |*args|
        arguments(args, required: [:job_id])
        do_get("jobs/#{job_id}/#{method}", query: arguments.params)
      end
    end

    def copy(*args)
      arguments(args, required: [:job_id])

      do_post("jobs/#{job_id}/copy", query: arguments.params)
    end

    def order(*args)
      arguments(args, required: [:job_id])

      do_post("jobs/#{job_id}/order", query: arguments.params)
    end

    def upload(*args)
      arguments(args, required: [:job_id]) do
        @params = args.last.read if args.last.is_a?(File)
      end
      do_put("jobs/#{job_id}/upload", arguments.params, headers: {'Content-Type' => 'text/csv'})
    end

    namespace :units, class_name: "Client::Units"
    namespace :worksets, class_name: "Client::Worksets"
  end
end
