module Crowdkit
  class Client::Poll < API
    include Enumerable

    def initialize(options, &block)
      super(options, &block)
      arguments([options], required: [:job_id])

      prepare_sawyer
      get_units

      @units.data.each do |unit|
        def unit.delete
          do_delete("units/#{id}/queue")
        end
      end unless @units.data.empty?
    end

    def each(&block)
      @units.each(&block)
    end

    def any?
      last_response.status != 304
    end

    def method_missing(method_name, *args, &block)
      @units.send(method_name, *args, &block)
    end

    def delete(*args)
      arguments(args, required: [:job_id])
      do_delete("jobs/#{job_id}/units/queue", {ids: @units.data.map(&:id)}) if any?
    end

    private

    def get_units
      @units = do_get("jobs/#{job_id}/units/queue", query: arguments.params)
    end

    def prepare_sawyer
      config_link = config
      Sawyer::Resource.send(:include, RequestMethods)
      Sawyer::Resource.send(:define_method, :config) { config_link }
    end
  end
end
