module Crowdkit
  class Client::Units < API
    def id_key
      :unit_id
    end

    def get(*args)
      arguments(args, required: [:unit_id])

      do_get("units/#{unit_id}")
    end
    alias :find :get
    alias :show :get

    def list(*args)
      arguments(args, required: [:job_id])

      do_get("jobs/#{job_id}/units", arguments.params)
    end
    alias :all :list

    def create(*args)
      arguments(args, required: [:job_id]) do
        @params = args.last if args.last.is_a?(Array)
      end

      do_post("jobs/#{job_id}/units", arguments.params)
    end

    def copy(*args)
      arguments(args, required: [:unit_id, :destination_job_id])

      do_post("units/#{unit_id}/copy", query: {destination_job_id: destination_job_id})
    end

    def delete(*args)
      arguments(args, required: [:unit_id])
      do_delete("units/#{unit_id}")
    end

    def judgments(*args)
      arguments(args, required: [:unit_id])
      do_get("units/#{unit_id}/judgments", query: arguments.params)
    end

    [:pause, :resume].each do |method|
      define_method(method) do |*args|
        arguments(args, required: [:unit_id])
        do_put("units/#{unit_id}/#{method}")
      end
    end

    namespace :poll, class_name: 'Client::Poll'
  end
end
