module Crowdkit
  class Config
    def self.config(key, value = nil, &block)
      @@config ||= {}
      attr_accessor key
      @@config[key] = {value: value, block: block}
    end

    config :access_token
    config :per_page
    config :debug, false
    config :adapter
    config :auto_paginate, false
    config :api_endpoint, "https://api.crowdflower.com/v2"
    config :user_agent, "Crowdkit Ruby Gem #{Crowdkit::VERSION}"
    config :ssl, {
      :ca_file => File.expand_path('../ssl_certs/cacerts.pem', __FILE__)
    }

    def initialize(overrides = {})
      @@config.each do |key,config|
        self.send("#{key}=", overrides[key] || ENV["CROWDKIT_#{key.upcase}"] || config[:value])
      end
      yield(self) if block_given?
    end

    def stack(&block)
      @stack ||= begin
        if block_given?
          Faraday::RackBuilder.new(&block)
        else
          Faraday::RackBuilder.new(&default_middleware)
        end
      end
    end

    def to_hash
      @@config.inject({}) do |memo,(key,value)|
        memo[key] = instance_variable_get(:"@#{key}")
        memo
      end
    end

    def connection_options
      {
        headers: { accept: "application/json", user_agent: self.user_agent },
        ssl: self.ssl
      }
    end

    protected

    def default_middleware
      proc do |builder|
        builder.response :logger if ENV['DEBUG'] || self.debug
        builder.use Faraday::Request::TokenAuthentication, self.access_token
        builder.use Crowdkit::Middleware::RaiseError
        builder.adapter self.adapter || Faraday.default_adapter
      end
    end
  end
end
